// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import clock from "clock";
import { HeartRateSensor } from "heart-rate";
import { charger, battery } from "power";
import document from "document";
import { logger, face } from "../common/utils";
import { tz_data, hr_tumbler } from "../common/tz.js";
import * as C from "../common/constants.js";
import * as names from "../common/names";
import * as utils from "../common/utils";
import * as prefs from "../common/prefs";

prefs.init();

face(document);

// Update the clock every second
clock.granularity = "seconds";

let hourHand = document.getElementById("hours");
let minHand = document.getElementById("mins");
let secHand = document.getElementById("secs");

let settings_enable = -1;

let pm = document.getElementById("pm");
let dayDate = document.getElementById("date");
let weekDay= document.getElementById("week");
const days = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"]

let tz1 = document.getElementById("tz1");
let tz1_entry = -4;
let tz1_index = names.get_info(tz1_entry).tz
tz1.text = "";

let tz2 = document.getElementById("tz2");
let tz2_entry = -2;
let tz2_index = names.get_info(tz2_entry).tz
tz2.text = "";
let time2 = document.getElementById("time2");

function set_rotation(r)
{

  set_data.rotation = r;
  let cc = document.getElementById("cc_btn");
  cc.image = ((r < 0) ? "ccw.png" : "cw.png");
  let min3 = document.getElementById("min-3");
  let min9 = document.getElementById("min-9");
  if (r < 0) {
    min3.text = "9";
    min9.text = "3";
  } else {
    min3.text = "3";
    min9.text = "9";
  }
}

let screens = [];
screens[C.SCR_FACE] = document.getElementById("analog_face");
screens[C.SCR_CFG0] = document.getElementById("config0");
screens[C.SCR_CFG1] = document.getElementById("config1");
screens[C.SCR_CFG2] = document.getElementById("config2");
screens[C.SCR_TZ1] = document.getElementById("custom_tz1");
screens[C.SCR_TZ2] = document.getElementById("custom_tz2");

let cfg_screen = C.SCR_CFG1;

let set_data = prefs.get();
tz1_entry = set_data.tz1;
tz2_entry = set_data.tz2;
tz1_index = names.get_info(tz1_entry).tz
tz2_index = names.get_info(tz2_entry).tz
set_rotation(set_data.rotation);

let bg = document.getElementById("click_bg");
bg.onclick = function(evt) {
  if (tz2_index >= 0) {
    let idx = tz1_index;
    tz1_index = tz2_index;
    tz2_index = idx;
  }
}

let batValue = document.getElementById("battery");

let bpmValue = document.getElementById("bpm");
let hrm = new HeartRateSensor();
hrm.onreading = function() {
  bpmValue.text = hrm.heartRate || "--";
  hrm.stop();
}

let url_text = document.getElementById("url_text");

function set_handler(tile, info)
{
  let touch = tile.getElementById("tz_touch");
  touch.onclick = evt => {
    if (names.get_info(info.index).tz == C.TZ_HEADER)
      return;
    let high = (info.cfg == C.SCR_CFG1) ? tz1_entry : tz2_entry;
    let otile = names.get_info(high).tile;
    if (otile != null)
      otile.getElementById("tz_bg").style.fill = "black";
    tile.getElementById("tz_bg").style.fill = "grey";
    if (info.cfg == C.SCR_CFG1) {
      tz1_entry = info.index;
      tz1_index = names.get_info(tz1_entry).tz;
    } else {
      tz2_entry = info.index;
      tz2_index = names.get_info(tz2_entry).tz;
    }
  };
}

function set_config(cfg_no)
{

  names.new_delegate();
  let disp = screens[cfg_no];
  let list = disp.getElementById("tz_names");
  list.delegate = {
    getTileInfo: function(index) {
      let idx = names.get_idx(cfg_no, index);
      let name = names.get_info(idx).name;
      return {
        type: "tz-pool",
        name: name,
        index: idx,
        cfg: cfg_no
      };
    },
    configureTile: function(tile, info) {
      if (info.type == "tz-pool") {
        let hi = (info.cfg == C.SCR_CFG1) ? tz1_entry : tz2_entry;
        tile.getElementById("tz_name").text = info.name;
        let bg = tile.getElementById("tz_bg");

        let tname = tile["id"];
        let id = names.get_index(info.cfg, tname);
        if (id != null) {
          names.set_tile(id, null);
        }
        names.set_index(info.cfg, tname, info.index);
        names.set_tile(info.index, tile);

        if (names.get_info(info.index).tz == C.TZ_HEADER)
          bg.style.fill = "slateblue";
        else
          bg.style.fill = (info.index == hi) ? "grey" : "black";

        set_handler(tile, info);
      }
    }
  }
  list.length = names.num_cities() + names.num_countries() + 2;
}

function sh_screen(idx)
{

  let max = screens.length;
  for (let i = 0; i < max; i++)
    screens[i].style.display = "none";
  screens[idx].style.display = "inline";
  if (idx == C.SCR_CFG1 || idx == C.SCR_CFG2)
    set_config(idx);
}

//
// Button to bring up the detail page
//
let listBtn = document.getElementById("list_btn");
listBtn.onmousedown = function(evt) {
  let local = new Date();
  settings_enable = local.getSeconds();
}
listBtn.onmouseup = function(evt) {
  settings_enable = -1;
}
function upd_settings() {
  if (settings_enable >= 0) {
    let local = new Date();
    if (local.getSeconds() != settings_enable) {
      settings_enable = -1;
      sh_screen(cfg_screen);
    }
  }
}

function sh_main() {

  set_data = prefs.save(tz1_entry, tz2_entry,
                        tz_data[35].off, tz_data[35].off,
                        set_data.rotation,
                        set_data.ampm, set_data.hr24);
  sh_screen(C.SCR_FACE);
}

let cc_btn = screens[C.SCR_CFG0].getElementById("cc_btn")
cc_btn.onclick = function(evt)
{

  set_rotation(-set_data.rotation);
}

let custom_btn1 = screens[C.SCR_CFG0].getElementById("btn_tz1");
custom_btn1.onclick = function(evt)
{   
      
  sh_screen(C.SCR_TZ1);
}

let custom_yes1 = screens[C.SCR_TZ1].getElementById("custom_yes");
custom_yes1.onclick = function(evt)
{   
      
  let hr_tumb = screens[C.SCR_TZ1].getElementById("hr_tumbler");
  let hr = hr_tumbler[hr_tumb.value];
  let min_tumb = screens[C.SCR_TZ1].getElementById("min_tumbler");
  let min = min_tumb.value * 15;
  tz_data[34].off = (hr * 60) + min;
  sh_screen(C.SCR_CFG0);
}

let custom_no1 = screens[C.SCR_TZ1].getElementById("custom_no");
custom_no1.onclick = function(evt)
{   
      
  sh_screen(C.SCR_CFG0);
}

let custom_btn2 = screens[C.SCR_CFG0].getElementById("btn_tz2");
custom_btn2.onclick = function(evt)
{   
      
  sh_screen(C.SCR_TZ2);
}

let custom_yes2 = screens[C.SCR_TZ2].getElementById("custom_yes");
custom_yes2.onclick = function(evt)
{   
      
  let hr_tumb = screens[C.SCR_TZ2].getElementById("hr_tumbler");
  let hr = hr_tumbler[hr_tumb.value];
  let min_tumb = screens[C.SCR_TZ2].getElementById("min_tumbler");
  let min = min_tumb.value * 15;
  tz_data[35].off = (hr * 60) + min;
  sh_screen(C.SCR_CFG0);
}

let custom_no2 = screens[C.SCR_TZ2].getElementById("custom_no");
custom_no2.onclick = function(evt)
{   
      
  sh_screen(C.SCR_CFG0);
}

//
// Buttons to go back to clock face
//
let clockBtn0 = screens[C.SCR_CFG0].getElementById("lr_btn");
clockBtn0.onclick = function(evt) {

  set_data.ampm = utils.get_ampm(screens[C.SCR_CFG0]);
  let btn = screens[C.SCR_CFG0].getElementById("cfg_hr24");
  set_data.hr24 = ((btn.value == 0) ? 1 : 0);
  sh_main();
}
let clockBtn1 = screens[C.SCR_CFG1].getElementById("lr_btn");
clockBtn1.onclick = function(evt) {

  sh_main();
}
let clockBtn2 = screens[C.SCR_CFG2].getElementById("lr_btn");
clockBtn2.onclick = function(evt) {

  sh_main();
}

//
// Button to go to timezone 1 from cfg
//
let cfg_2_tz1 = screens[C.SCR_CFG0].getElementById("ur_btn");
cfg_2_tz1.onclick = function(evt) {
  cfg_screen = C.SCR_CFG1;
  set_data.ampm = utils.get_ampm(screens[C.SCR_CFG0]);
  let btn = screens[C.SCR_CFG0].getElementById("cfg_hr24");
  set_data.hr24 = ((btn.value == 0) ? 1 : 0);
  sh_screen(cfg_screen);
}

//
// Button to go to config from tz1
//
let tz1_2_cfg = screens[C.SCR_CFG1].getElementById("ul_btn");
tz1_2_cfg.onclick = function(evt) {
  cfg_screen = C.SCR_CFG0;
  let btn = screens[C.SCR_CFG0].getElementById("cfg_ampm" + set_data.ampm);
  btn.value=1;
  let btn = screens[C.SCR_CFG0].getElementById(
              (set_data.hr24 == 1) ? "cfg_hr12" : "cfg_hr24");
  btn.value=1;
  sh_screen(cfg_screen);
}

//
// Button to go to tz2 from tz1
//
let tz1_2_tz2 = screens[C.SCR_CFG1].getElementById("ur_btn");
tz1_2_tz2.onclick = function(evt) {
  cfg_screen = C.SCR_CFG2;
  sh_screen(cfg_screen);
}

//
// Button to go to tz1 from tz2
//
let tz1_2_tz2 = screens[C.SCR_CFG2].getElementById("ul_btn");
tz1_2_tz2.onclick = function(evt) {
  cfg_screen = C.SCR_CFG1;
  sh_screen(cfg_screen);
}

// Returns an angle (0-360) for the current hour in the day, including minutes
function hoursToAngle(hours, minutes) {
  let hourAngle = (360 / 12) * hours;
  let minAngle = (360 / 12 / 60) * minutes;
  return (hourAngle + minAngle) * set_data.rotation;
}

// Returns an angle (0-360) for minutes
function minutesToAngle(minutes) {
  return ((360 / 60) * minutes) * set_data.rotation;
}

// Returns an angle (0-360) for seconds
function secondsToAngle(seconds) {
  return ((360 / 60) * seconds) * set_data.rotation;
}

function updateBattery() {
  
  let lvl = Math.floor(battery.chargeLevel);
  batValue.text = lvl;
  if (lvl > 50)
    batValue.style.fill = "lime";
  else if (lvl > 25)
    batValue.style.fill = "yellow";
  else
    batValue.style.fill = "red";
}

function updateBpm() {
  
}

// Rotate the hands every tick
function updateClock() {

  let local = new Date();
  let tm1 = local;
  let tm2;
  if (tz1_index > 0 || tz2_index >= 0) {
    let off = utils.utc_off(tm1.toString());
    let utc_time = utils.utc(tm1, -off);
    if (tz1_index > 0)
      tm1 = utils.utc(utc_time, utils.tz_off(tz1_index, utc_time));
    if (tz2_index > 0)
      tm2 = utils.utc(utc_time, utils.tz_off(tz2_index, utc_time));
    else if (tz2_index == 0)
      tm2 = local;
  }
  let hours = tm1.getHours();
  let mins = tm1.getMinutes();
  let secs = tm1.getSeconds();

  pm.text = utils.ampm(hours, set_data, "AM", "PM");

  hourHand.groupTransform.rotate.angle = hoursToAngle(hours%12, mins);
  minHand.groupTransform.rotate.angle = minutesToAngle(mins);
  secHand.groupTransform.rotate.angle = secondsToAngle(secs);

  tz1.text = ((tz1_index == 0) ? "" : utils.find_tz(tz1_index, utc_time));

  let tz = "";
  let time = "";
  if (tz2_index >= 0) {
    tz = utils.find_tz(tz2_index, utc_time);
    let hr = tm2.getHours();
    if (set_data.hr24 == 1 && hr > 12)
      hr -= 12;
    let mn = tm2.getMinutes();
    time = utils.numPad(hr, " ") + ":" + utils.numPad(mn, "0");
    if (set_data.hr24 != 0)
      time += utils.ampm(hours, set_data, " A", " P");
  }
  tz2.text = tz;
  time2.text = time;

  dayDate.text = tm1.getDate();
  weekDay.text = days[tm1.getDay()];
  
  updateBattery();
  
  hrm.start();

  upd_settings();
}

// Update the clock every tick event
clock.ontick = () => updateClock();
