// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import { tz_data, tz_day } from "../common/tz.js";
import { cities } from "../common/cities.js"
import { countries } from "../common/countries.js"

export function logger(str)
{

  console.log("DDD: " + str);
}

export function pr_object(object) {
  for(var key in object) {
    try {
      logger('  Key: ' + key + ' | value: ' + object[key]);
    } catch (error) {
      // Some values throw an error when trying to access them.
      logger('  Key: ' + key + ' | Error: ' + error.message);
    }
  }
}

// Add zero in front of numbers < 10
export function numPad(i, p) {
  if (i < 10) {
    i = p + i;
  }
  return i;
}

export function sw_screen(s1, s2)
{
  s1.style.display = "inline";
  s2.style.display = "none";
}

export function face(doc)
{
  let bar;

  for (let i = 0; i < 12; i++) {
    bar = doc.getElementById("tick-" + i);
    bar.groupTransform.rotate.angle = (i * 30);
  }
}

export function utc(time, off)
{

  let yr = time.getFullYear();
  let mo = time.getMonth();
  let day = time.getDate();
  let hr = time.getHours();
  let mn = time.getMinutes();
  let sec = time.getSeconds();

  let d = 0;
  let min = ((hr * 60) + mn) + off;
  if (min < 0) {
    d = -1;
    min += (24 * 60);
  } else if (min >= (24 * 60)) {
    d = 1;
    min -= (24 * 60);
  }
  hr = parseInt(min / 60);
  min -= (hr * 60);
  return new Date(yr, mo, day, hr, min, sec);
}

export function utc_off(gmt)
{
  let sign;

  gmt = gmt.substring(gmt.indexOf("GMT") + 3);
  let sign = gmt.charAt(0);
  if (gmt.charAt(0) == '-')
    gmt = gmt.substring(1);
  let idx = gmt.indexOf(":");
  let h = parseInt(gmt.substring(0, idx));
  let m = parseInt(gmt.substring(idx + 1));
  let r = (h * 60) + m;
  return (sign == '-') ? -r : r;
}

const mo_days = [ 31, 27, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ];
function find_day(day, month, num, year)
{
  let start;

  let ndays = mo_days[month];
  let time = new Date(year, month);
  let first = time.getDay();
  first = 1 - (day - first) - 7;
  if (num > 0)
    return first + (7 * num);
  while ((day + 7) <= ndays)
    day += 7;
  return day;
}

function dst(idx, time)
{
  let mo_day;

  if (tz_data[idx].days == null)
     return false;
  let days = tz_data[idx].days;
  let month = time.getMonth();
  let day = time.getDate();
  if (month < days.st_mo || month > days.st_mo)
    return false;
  if (month == days.st_mo) {
    mo_day = find_day(days.st_day, days.st_mo, days.st_num, time.getFullYear());
    if (day < mo_day)
      return false;
    else if (day > mo_day)
      return true;
return true;  // DDD - calculate day based on hour
  } else if (month == days.end_mo) {
    mo_day = find_day(days.end_day, days.end_mo, days.end_num, time.getFullYear());
    if (day > mo_day)
      return false;
    else if (day < mo_day)
      return true;
return false;  // DDD - calculate day based on hour
  }
  return true;
}

export function find_tz(idx, utc)
{

  if (dst(idx, utc))
    return tz_data[idx].daylight;
  return tz_data[idx].standard;
}

export function tz_off(idx, utc)
{

  let off = tz_data[idx].off;
  if (dst(idx, utc))
    off += 60;
  return off;
}

export function ampm(h, pref, am, pm)
{

  let show = pref.ampm;
  if (pref.hr24 == 0)
    show = 1;
  switch (show) {

  case 0:
    return "";

  case 1:
    return (h > 12) ? pm : am;

  case 2:
    return (h > 12) ? pm : "";

  }
}

export function get_ampm(w)
{

  let btn = w.getElementById("cfg_ampm0");
  if (btn.value != 0)
    return 0;
  btn = w.getElementById("cfg_ampm1");
  if (btn.value != 0)
    return 1;
  btn = w.getElementById("cfg_ampm2");
  if (btn.value != 0)
    return 2;
}
